package com.cashfree.cashfree_pg;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import androidx.annotation.NonNull;

import com.gocashfree.cashfreesdk.CFPaymentService;

import java.util.HashMap;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

public class CashfreePgPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  private MethodChannel channel;
  private Activity activity;
  private Result flutterResult;

  private CashfreePgPlugin(Activity activity) {
    this.activity = activity;
  }

  public CashfreePgPlugin() {
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "cashfree_pg");
    channel.setMethodCallHandler(this);
  }

  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "cashfree_pg");
    channel.setMethodCallHandler(new CashfreePgPlugin());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    this.flutterResult = result;
    if (call.method.equals("doPayment")) {
      try {
        CFPaymentService cfPaymentServiceInstance = CFPaymentService.getCFPaymentServiceInstance();
        if (call.arguments != null) {
          Map<String, Object> map = (Map<String, Object>) call.arguments;
          Map<String, String> params = new HashMap<String, String>();
          for (Map.Entry<String, Object> entry : map.entrySet()) {
            if (entry.getValue() instanceof String) {
              params.put(entry.getKey(), String.valueOf(entry.getValue()));
              Log.d(entry.getKey(), String.valueOf(entry.getValue()));
            }
          }
          boolean hideOrderId = params.containsKey("hideOrderId")? Boolean.valueOf("hideOrderId") : false;
          if (params.containsKey("color1") && params.containsKey("color2")) {
            Log.d("TAG",params.get("color1") + "\t" +params.get("color2") );
            cfPaymentServiceInstance.doPayment(this.activity, params, params.get("tokenData"), params.get("stage"), params.get("color1"),params.get("color2"), hideOrderId);
          } else {
            Log.d("TAG","Theme color not present in input params using default" );
            cfPaymentServiceInstance.doPayment(this.activity, params, params.get("tokenData"), params.get("stage"), "#784BD2", "#FFFFFF", hideOrderId);
          }
        } else {
          sendFailedStatus();
        }
      }catch(Exception e){
        sendFailedStatus();
        e.printStackTrace();
      }
    } else {
      sendFailedStatus();
      result.notImplemented();
    }
  }

  private void sendFailedStatus() {
    Map<String, String> resultMap = new HashMap();
    resultMap.put("txStatus", "FAILED");
    resultMap.put("txMsg", "SDK Internal Error");
    flutterResult.success(resultMap);
  }


  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    this.activity = binding.getActivity();
    binding.addActivityResultListener(new PluginRegistry.ActivityResultListener() {
      public boolean onActivityResult(int i, int i1, Intent data) {
        if (data != null) {
          Map<String, String> resultMap = new HashMap<String, String>();
          Bundle bundle = data.getExtras();
          if (bundle != null) {
            for (String key : bundle.keySet()) {
              if (bundle.getString(key) != null) {
                resultMap.put(key, bundle.getString(key));
              }
            }
          }
          flutterResult.success(resultMap);
        } else {
          sendFailedStatus();
        }
        return false;
      }
    });
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(ActivityPluginBinding activityPluginBinding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }
}
