//
//  BaseViewController.swift
//  CFSDK-iOS
//
//  Created by Arjun Seshadhry on 03/09/19.
//  Copyright © 2019 CashFree. All rights reserved.
//

import Foundation
import WebKit

public protocol ResultDelegate {
    func onPaymentCompletion(msg: String)
}

public class CFViewController : UIViewController {
    var inputParams: Dictionary<String, Any>

    private var appId_config = "" as NSString
    private var env_config = "" as NSString

    private func setConfig(env: String, appId: String) {
        env_config = env as NSString
        appId_config = appId as NSString
    }

    public init(params: Dictionary <String, Any>, appId: String, env: String, callBack: ResultDelegate) {
        self.inputParams = params
        self.resultDelegate = callBack
        super.init(nibName: nil, bundle: nil)
        self.setConfig(env: env, appId: appId)
        modalPresentationStyle = .fullScreen
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder: ) has not been implemented")
    }
    
    private var resultDelegate: ResultDelegate?
    
    let CFWebView : WKWebView = {
        let webView = WKWebView()
        webView.backgroundColor = UIColor.white
        return webView
    }()
    
    @objc private func cancel(sender: UIView ) {
            let refreshAlert = UIAlertController(title: "Cancel", message: "Are you sure you want to cancel the payment?", preferredStyle: UIAlertController.Style.alert)
            refreshAlert.addAction(UIAlertAction(title: "Ok", style: .default, handler: { (action: UIAlertAction!) in
                self.resultSent = true
                self.dismiss(animated: true, completion: nil)
                self.resultDelegate?.onPaymentCompletion(msg: "{\"txStatus\":\"CANCELLED\", \"txMsg\":\"Cancelled by user\"}")
            }))
            refreshAlert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: { (action: UIAlertAction!) in
                
            }))
            
            present(refreshAlert, animated: true, completion: nil)
    }
    func statusBarHeight() -> CGFloat {
        let statusBarSize = UIApplication.shared.statusBarFrame.size
        return Swift.min(statusBarSize.width, statusBarSize.height)
    }
    private func paySecurely_request() {
        let cardPaymentFormHtml = CFPaymentFormBuilder().createForm(stage: env_config as String, appId: appId_config as String, inputParams: inputParams)
        self.CFWebView.loadHTMLString(cardPaymentFormHtml, baseURL: nil)
    }
    
    public override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        if isBeingDismissed && !self.resultSent {
            self.resultDelegate?.onPaymentCompletion(msg: "{\"txStatus\":\"CANCELLED\", \"txMsg\":\"Cancelled by user\"}")
        }
    }
    public override func viewDidLoad() {
        super.viewDidLoad()
        self.view.translatesAutoresizingMaskIntoConstraints = false
        self.view.backgroundColor = .white
        
        CFWebView.frame = self.view.bounds
        CFWebView.navigationDelegate = self
        self.view.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(CFWebView)
        
        let stview = UIView(frame: CGRect(
        x: 0,
        y: 0,
        width: self.view.bounds.width,
        height: 20))
        stview.backgroundColor=UIColor(rgb: 0x784BD2);
        view.addSubview(stview)
        CFWebView.scrollView.contentInset = UIEdgeInsets(top: 70, left: 0, bottom: 0, right: 0);
        
        let topLayout = UIView(frame: CGRect(
        x: 0,
        y: 20,
        width: self.view.bounds.width,
        height: 50))
        
        topLayout.backgroundColor = UIColor(rgb: 0x784BD2)
        view.addSubview(topLayout)
        
        let orderIdLabel = MyLabel(frame: CGRect(
            x: (CFWebView.scrollView.frame.size.width)/3,
            y: 0,
            width: (CFWebView.scrollView.frame.size.width) - (2 * (CFWebView.scrollView.frame.size.width)/3),
            height: 50))
        orderIdLabel.font = orderIdLabel.font.withSize(15)
        orderIdLabel.font.fontDescriptor.withSymbolicTraits(UIFontDescriptor.SymbolicTraits(arrayLiteral: .traitBold))
        orderIdLabel.textColor = UIColor(rgb:  0xFFFFFF)
        orderIdLabel.text = "#\(inputParams["orderId"] ?? "")"
        orderIdLabel.adjustsFontSizeToFitWidth = true
        orderIdLabel.numberOfLines=1
        topLayout.addSubview(orderIdLabel)
        
        
        let backButton = UIButton(frame: CGRect(
        x: 10,
        y: 0,
        width: 40,
        height: 50))
        backButton.setTitle("Back", for: .normal)
        backButton.titleLabel!.font = backButton.titleLabel!.font.withSize(15)
        //"#784BD2", "#FFFFFF"
        backButton.setTitleColor(UIColor(rgb:  0xFFFFFF), for: .normal)
        backButton.addTarget(self, action:#selector(goBack(sender:)), for: .touchUpInside)
        topLayout.addSubview(backButton)
        let cancelButton = UIButton(frame: CGRect(
        x: (CFWebView.scrollView.frame.size.width) - 60,
        y: 0,
        width: 50,
        height: 50))
        cancelButton.titleLabel!.font = cancelButton.titleLabel!.font.withSize(15)
        cancelButton.addTarget(self, action:#selector(cancel(sender:)), for: .touchUpInside)
        cancelButton.setTitle("Cancel", for: .normal)
        cancelButton.setTitleColor(UIColor(rgb:  0xFFFFFF), for: .normal)
        topLayout.addSubview(cancelButton)
        topLayout.layer.shadowOffset = CGSize(width: 0, height: 3)
        topLayout.layer.shadowRadius = 1
        topLayout.layer.shadowOpacity = 0.5
        topLayout.layer.shadowColor = UIColor.gray.cgColor
        topLayout.layer.masksToBounds = false
        paySecurely_request()
    }

    @objc func goBack(sender: UIView) {
        if(!CFWebView.canGoBack) {
            cancel(sender: sender)
        } else {
            let refreshAlert = UIAlertController(title: "Cancel", message: "Are you sure you want to go back?", preferredStyle: UIAlertController.Style.alert)
            refreshAlert.addAction(UIAlertAction(title: "OK", style: .default, handler: { (action: UIAlertAction!) in
                let historySize = self.CFWebView.backForwardList.backList.count
                let firstItem = self.CFWebView.backForwardList.item(at: -historySize)
                self.CFWebView.go(to: firstItem!)
            }))
            refreshAlert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: { (action: UIAlertAction!) in
                
            }))
            
            present(refreshAlert, animated: true, completion: nil) }
    }
    var resultSent = false;
}
extension CFViewController: WKNavigationDelegate {
    public func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        if (navigationAction.navigationType == .linkActivated) {
            decisionHandler(.cancel)
        } else {
            decisionHandler(.allow)
        }
        
        let encodedJsonStr = navigationAction.request.url?.fragment?.removingPercentEncoding
        let iosUrlScheme = navigationAction.request.url?.scheme;
        
        guard let CFUrlScheme = iosUrlScheme else { return }
        
        switch CFUrlScheme {
        case "https":
            print("https")
        case "custom-mob-sdk":
            if encodedJsonStr != nil {
                self.resultSent = true
                self.dismiss(animated: true, completion: nil)
                resultDelegate?.onPaymentCompletion(msg: String(encodedJsonStr!))
            }
            break
        default:
            break
        }
    }
    
    public func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        print("Finished navigating to url \(webView.url!)");
        CFWebView.scrollView.translatesAutoresizingMaskIntoConstraints = false
            CFWebView.scrollView.contentSize = CGSize(
                width: CFWebView.scrollView.bounds.width,
                height: CFWebView.frame.height + 50);
    }
}
extension UIColor {
   convenience init(red: Int, green: Int, blue: Int) {
       assert(red >= 0 && red <= 255, "Invalid red component")
       assert(green >= 0 && green <= 255, "Invalid green component")
       assert(blue >= 0 && blue <= 255, "Invalid blue component")

       self.init(red: CGFloat(red) / 255.0, green: CGFloat(green) / 255.0, blue: CGFloat(blue) / 255.0, alpha: 1.0)
   }

   convenience init(rgb: Int) {
       self.init(
           red: (rgb >> 16) & 0xFF,
           green: (rgb >> 8) & 0xFF,
           blue: rgb & 0xFF
       )
   }
}

class MyLabel : UILabel {
    override func drawText(in rect: CGRect) {
        let insets = UIEdgeInsets(top: 5, left: 15, bottom: 0, right: 15)
//                super.drawText(in: UIEdgeInsetsInsetRect(rect, insets))
        super.drawText(in: rect.inset(by: insets))
    }
}
